=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-07-11 07:05:34 +0000
+++ bzrlib/branch.py	2005-07-11 16:59:50 +0000
@@ -27,7 +27,6 @@
 ## TODO: Maybe include checks for common corruption of newlines, etc?
 
 
-
 def find_branch(f, **args):
     if f and (f.startswith('http://') or f.startswith('https://')):
         import remotebranch 

=== added file 'bzrlib/local_transport.py'
--- bzrlib/local_transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/local_transport.py	2005-07-11 16:59:50 +0000
@@ -0,0 +1,73 @@
+#!/usr/bin/env python
+"""\
+An implementation of the Transport object for local
+filesystem access.
+"""
+
+from bzrlib.transport import Transport, protocol_handlers
+import os
+
+class LocalTransport(Transport):
+    """This is the transport agent for local filesystem access."""
+
+    def __init__(self, base):
+        """Set the base path where files will be stored."""
+        from os.path import realpath
+        super(LocalTransport, self).__init__(realpath(base))
+
+    def _join(self, relpath):
+        return os.path.join(self.base, relpath)
+
+    def has(self, relpath):
+        return os.access(self._join(relpath), os.F_OK)
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+        """
+        return open(self._join(relpath), 'rb')
+
+    def put(self, relpath, f):
+        """Copy the file-like object into the location.
+        """
+        from bzrlib.atomicfile import AtomicFile
+
+        fp = AtomicFile(self._join(relpath), 'wb')
+        self._pump(f, fp)
+        fp.commit()
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        fp = open(self._join(relpath), 'a+b')
+        self._pump(f, fp)
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        import shutil
+        path_from = self._join(rel_from)
+        path_to = self._join(rel_to)
+        shutil.copy(path_from, path_to)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        path_from = self._join(rel_from)
+        path_to = self._join(rel_to)
+
+        os.rename(path_from, path_to)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        os.remove(self._join(relpath))
+
+    def async_get(self, relpath):
+        """Make a request for an file at the given location, but
+        don't worry about actually getting it yet.
+
+        :rtype: AsyncFile
+        """
+        raise NotImplementedError
+
+# If nothing else matches, try the LocalTransport
+protocol_handlers[None] = LocalTransport
+

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-06-27 01:36:22 +0000
+++ bzrlib/store.py	2005-07-11 16:59:50 +0000
@@ -21,7 +21,7 @@
 unique ID.
 """
 
-import os, tempfile, types, osutils, gzip, errno
+import os, tempfile, osutils, gzip, errno
 from stat import ST_SIZE
 from StringIO import StringIO
 from trace import mutter
@@ -32,15 +32,82 @@
 class StoreError(Exception):
     pass
 
-
-class ImmutableStore(object):
+class Storage(object):
+    """This class represents the abstract storage layout for saving information.
+    """
+    def __init__(self, transport):
+        self._transport = transport
+        self._max_buffered_requests = 10
+
+    def __repr__(self):
+        return "%s(%r)" % (self.__class__.__name__, self._transport.base)
+
+    __str__ == __repr__
+
+    def __len__(self):
+        raise NotImplementedError('Children should define their length')
+
+    def __getitem__(self, fileid):
+        """Returns a file reading from a particular entry."""
+        raise NotImplementedError
+
+    def __contains__(self, fileid):
+        """"""
+        raise NotImplementedError
+
+    def __iter__(self):
+        raise NotImplementedError
+
+    def add(self, f, fileid):
+        """Add a file object f to the store accessible from the given fileid"""
+        raise NotImplementedError('Children of Storage must define their method of adding entries.')
+
+    def copy_multi(self, other, ids):
+        """Copy texts for ids from other into self.
+
+        If an id is present in self, it is skipped.  A count of copied
+        ids is returned, which may be less than len(ids).
+        """
+        from bzrlib.progress import ProgressBar
+        pb = ProgressBar()
+        pb.update('preparing to copy')
+        to_copy = [fileid for fileid in ids if fileid not in self]
+        return self._do_copy(other, to_copy, pb)
+
+    def _do_copy(self, other, to_copy, pb):
+        """This is the standard copying mechanism, just get them one at
+        a time from remote, and store them locally.
+        """
+        count = 0
+        buffered_requests = []
+        for fileid in to_copy:
+            buffered_requests.append((other[fileid], fileid))
+            if len(buffered_requests) > self._max_buffered_requests:
+                self.add(*buffered_requests.pop(0))
+                count += 1
+                pb.update('copy', count, len(to_copy))
+
+        for req in buffered_requests:
+            self.add(*req)
+            count += 1
+            pb.update('copy', count, len(to_copy))
+
+        assert count == len(to_copy)
+        pb.clear()
+        return count
+
+
+
+class CompressedTextStore(Storage):
     """Store that holds files indexed by unique names.
 
     Files can be added, but not modified once they are in.  Typically
     the hash is used as the name, or something else known to be unique,
     such as a UUID.
 
-    >>> st = ImmutableScratchStore()
+    Files are stored gzip compressed, with no delta compression.
+
+    >>> st = ScratchFlatTextStore()
 
     >>> st.add(StringIO('hello'), 'aa')
     >>> 'aa' in st
@@ -64,15 +131,15 @@
     """
 
     def __init__(self, basedir):
-        self._basedir = basedir
+        super(CompressedTextStore, self).__init__(basedir)
 
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
+    def _path(self, fileid):
+        if '\\' in fileid or '/' in fileid:
+            raise ValueError("invalid store id %r" % fileid)
+        return self._transport.get_filename(fileid)
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self._basedir)
+        return "%s(%r)" % (self.__class__.__name__, self._location)
 
     def add(self, f, fileid, compressed=True):
         """Add contents of a file into the store.
@@ -80,58 +147,38 @@
         f -- An open file, or file-like object."""
         # FIXME: Only works on files that will fit in memory
         
-        from bzrlib.atomicfile import AtomicFile
+        from cStringIO import StringIO
         
         mutter("add store entry %r" % (fileid))
-        if isinstance(f, types.StringTypes):
+        if isinstance(f, basestring):
             content = f
         else:
             content = f.read()
             
-        p = self._path(fileid)
-        if os.access(p, os.F_OK) or os.access(p + '.gz', os.F_OK):
-            raise BzrError("store %r already contains id %r" % (self._basedir, fileid))
+        if self._transport.has(fileid) or self._transport.has(fileid + '.gz'):
+            raise BzrError("store %r already contains id %r" % (self._location, fileid))
 
-        fn = p
+        fn = fileid
         if compressed:
             fn = fn + '.gz'
             
-        af = AtomicFile(fn, 'wb')
-        try:
-            if compressed:
-                gf = gzip.GzipFile(mode='wb', fileobj=af)
-                gf.write(content)
-                gf.close()
-            else:
-                af.write(content)
-            af.commit()
-        finally:
-            af.close()
-
-
-    def copy_multi(self, other, ids):
-        """Copy texts for ids from other into self.
-
-        If an id is present in self, it is skipped.  A count of copied
-        ids is returned, which may be less than len(ids).
-        """
-        from bzrlib.progress import ProgressBar
-        pb = ProgressBar()
-        pb.update('preparing to copy')
-        to_copy = [id for id in ids if id not in self]
-        if isinstance(other, ImmutableStore):
-            return self.copy_multi_immutable(other, to_copy, pb)
-        count = 0
-        for id in to_copy:
-            count += 1
-            pb.update('copy', count, len(to_copy))
-            self.add(other[id], id)
-        assert count == len(to_copy)
-        pb.clear()
-        return count
-
-
-    def copy_multi_immutable(self, other, to_copy, pb):
+        sio = StringIO()
+        if compressed:
+            gf = gzip.GzipFile(mode='wb', fileobj=sio)
+            gf.write(content)
+            gf.close()
+        else:
+            sio.write(content)
+        sio.seek(0)
+        self._transport.put(fn, sio)
+
+    def _do_copy(self, other, to_copy, pb):
+        if isinstance(other, CompressedTextStore):
+            return self._copy_multi_text(other, to_copy, pb)
+        return super(CompressedTextStore, self)._do_copy(other, to_copy, pb)
+
+
+    def _copy_multi_text(self, other, to_copy, pb):
         from shutil import copyfile
         count = 0
         for id in to_copy:
@@ -161,7 +208,7 @@
     # TODO: Guard against the same thing being stored twice, compressed and uncompresse
 
     def __iter__(self):
-        for f in os.listdir(self._basedir):
+        for f in os.listdir(self._location):
             if f[-3:] == '.gz':
                 # TODO: case-insensitive?
                 yield f[:-3]
@@ -169,7 +216,7 @@
                 yield f
 
     def __len__(self):
-        return len(os.listdir(self._basedir))
+        return len(os.listdir(self._location))
 
     def __getitem__(self, fileid):
         """Returns a file reading from a particular entry."""
@@ -202,20 +249,20 @@
 
 
 
-class ImmutableScratchStore(ImmutableStore):
+class ScratchFlatTextStore(CompressedTextStore):
     """Self-destructing test subclass of ImmutableStore.
 
     The Store only exists for the lifetime of the Python object.
- Obviously you should not put anything precious in it.
+    Obviously you should not put anything precious in it.
     """
     def __init__(self):
-        ImmutableStore.__init__(self, tempfile.mkdtemp())
+        super(ScratchFlatTextStore, self).__init__(tempfile.mkdtemp())
 
     def __del__(self):
-        for f in os.listdir(self._basedir):
-            fpath = os.path.join(self._basedir, f)
+        for f in os.listdir(self._location):
+            fpath = os.path.join(self._location, f)
             # needed on windows, and maybe some other filesystems
             os.chmod(fpath, 0600)
             os.remove(fpath)
-        os.rmdir(self._basedir)
+        os.rmdir(self._location)
         mutter("%r destroyed" % self)

=== added file 'bzrlib/transport.py'
--- bzrlib/transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport.py	2005-07-11 16:59:50 +0000
@@ -0,0 +1,211 @@
+#!/usr/bin/env python
+"""\
+This module contains the basic class handling transport of
+information.
+"""
+
+protocol_handlers = {
+}
+
+class AsyncError(Exception):
+    pass
+
+class AsyncFile(object):
+    """This will be returned from a Transport object,
+    whenever an asyncronous get is requested.
+    """
+    _max_write_buffer = 8192
+
+    def __init__(self):
+        self.read_buffer = []
+        self.write_buffer = []
+        self._finalized = False
+
+    def _total_len(self, buffer):
+        count = 0
+        for b in buffer:
+            count += len(b)
+        return count
+
+    def finalize(self):
+        """This will block until all data has been buffered for
+        reading, or all data has been flushed for writing.
+        Once this is called, you can no longer write more data.
+        """
+        raise NotImplementedError
+
+    def can_read(self):
+        """Can we read some data without blocking"""
+        return len(self.read_buffer) > 0
+
+    def can_write(self):
+        """Can we write some more data out without blocking?"""
+        return (not self._finalized \
+                and self._total_len(self.write_buffer) < self._max_write_buffer)
+
+    def write(self, chunk):
+        if self._finalized:
+            raise AsyncError('write attempted on finalized file.')
+        self.write_buffer.append(chunk)
+
+    def read(self, size=None):
+        if size is None:
+            return ''.join(self.read_buffer)
+        else:
+            out = ''
+            while True:
+                buf = self.read_buffer.pop(0)
+                if len(buf) + len(out) < size:
+                    out += buf
+                else:
+                    left = size - len(out)
+                    out += buf[:left]
+                    buf = buf[left:]
+                    self.read_buffer.insert(0, buf)
+                    return out
+
+
+class Transport(object):
+    """This class encapsulates methods for retrieving or putting a file
+    from/to a storage location.
+
+    Most functions have a _multi variant, which allows you to queue up
+    multiple requests. They generally have a dumb base implementation 
+    which just iterates over the arguments, but smart Transport
+    implementations can do pipelining.
+    """
+
+    def __init__(self, base):
+        self.base = base
+
+    def _pump(self, from_file, to_file):
+        """Most children will need to copy from one file-like 
+        object or string to another one.
+        This just gives them something easy to call.
+        """
+        if isinstance(from_file, basestring):
+            to_file.write(basestring)
+        else:
+            from bzrlib.osutils import pumpfile
+            pumpfile(from_file, to_file)
+
+    def has(self, relpath):
+        """Does the target location exist?"""
+        raise NotImplementedError
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+        """
+        raise NotImplementedError
+
+    def get_multi(self, relpaths):
+        """Get a list of file-like objects, one for each entry in relpaths.
+
+        :param relpaths: A list of relative paths.
+        :return: A list of file-like objects.
+        """
+        files = []
+        for relpath in relpaths:
+            files.append(self.get(relpath))
+        return files
+
+    def put(self, relpath, f):
+        """Copy the file-like or string object into the location.
+        """
+        raise NotImplementedError
+
+    def put_multi(self, files):
+        """Put a set of files or strings into the location.
+
+        :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
+        """
+        for path, f in files:
+            self.put(path, f)
+
+    def append(self, relpath, f):
+        """Append the text in the file-like or string object to 
+        the supplied location.
+        """
+        raise NotImplementedError
+
+    def append_multi(self, files):
+        """Append the text in each file-like or string object to
+        the supplied location.
+        """
+        for path, f in files:
+            self.append(path, f)
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        raise NotImplementedError
+
+    def copy_multi(self, relpaths):
+        """Copy a bunch of entries.
+        
+        :param relpaths: A list of tuples of the form [(from, to), (from, to),...]
+        """
+        # This is the non-pipelined implementation, so that
+        # implementors don't have to implement everything.
+        for rel_from, rel_to in relpaths:
+            self.copy(rel_from, rel_to)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        raise NotImplementedError
+
+    def move_multi(self, relpaths):
+        """Move a bunch of entries.
+        
+        :param relpaths: A list of tuples of the form [(from1, to1), (from2, to2),...]
+        """
+        for rel_from, rel_to in relpaths:
+            self.move(rel_from, rel_to)
+
+    def move_multi_to(self, relpaths, rel_to):
+        """Move a bunch of entries to a single location.
+        This differs from move_multi in that you give a list of from, and
+        a single destination, rather than multiple destinations.
+
+        :param relpaths: A list of relative paths [from1, from2, from3, ...]
+        :param rel_to: A directory where each entry should be placed.
+        """
+        # This is not implemented, because you need to do special tricks to
+        # extract the basename, and add it to rel_to
+        raise NotImplementedError
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        raise NotImplementedError
+
+    def delete_multi(self, relpaths):
+        """Queue up a bunch of deletes to be done.
+        """
+        for relpath in relpaths:
+            self.delete(relpath)
+
+    def async_get(self, relpath):
+        """Make a request for an file at the given location, but
+        don't worry about actually getting it yet.
+
+        :rtype: AsyncFile
+        """
+        raise NotImplementedError
+
+    def list_dir(self, relpath):
+        """Return a list of all files at the given location.
+        WARNING: many transports do not support this, so trying avoid using
+        it if at all possible.
+        """
+        raise NotImplementedError
+
+def transport(base):
+    for proto, klass in protocol_handlers.iteritems():
+        if proto is not None and base.startswith(proto):
+            return klass(base)
+    # The default handler is the filesystem handler
+    # which has a lookup of None
+    return protocol_handlers[None](base)
+
+
+# Local transport should always be initialized
+import local_transport

